# appteka - helpers collection

# Copyright (C) 2018-2021 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Writing data to IO text buffer using queue and working in separate
thread."""

from collections import deque
from threading import Thread, Lock, Event
from time import sleep


class QueuedWriter:
    """Tool for record data to text buffer using queue and thread."""
    def __init__(self, buff=None, write_on='time', write_every=1):
        _check_write_on(write_on)

        self._buff = buff
        self._queue = deque([])
        self._thread = None
        self._thread_args = (self._save_queue, write_on, write_every)
        self._convert_func = lambda x: x

    def set_buff(self, buff):
        """Set IO buffer."""
        self._buff = buff

    def set_convert_func(self, func):
        """Set function for convertion data sample to string."""
        self._convert_func = func

    def start_thread(self):
        """Start recording thread."""
        self._thread = PyRecordThread(*self._thread_args)
        self._thread.start()

    def add_data(self, sample):
        """Add data sample."""
        self._queue.append(sample)
        self._thread.write_data()

    def _save_queue(self):
        """Move data from queue to buffer."""
        while self._queue:
            sample = self._queue.popleft()
            line = self._convert_func(sample)
            self._buff.write(line)

    def stop_thread(self):
        """Stop recording record."""
        self._thread.write_data()
        self._thread.stop()


class PyRecordThread(Thread):
    """Thread in which samples recorded to IO buffer."""
    def __init__(self, save_func, write_on='time', write_every=1):
        super().__init__()
        self._lock = Lock()
        self._must_write = False
        self._can_quit = Event()
        self._have_data = Event()
        self.save_func = save_func

        _check_write_on(write_on)

        if write_on == 'time':
            self._loop_func = self._sleep_loop
            self._loop_func_args = (write_every, )

        if write_on == 'data':
            self._loop_func = self._event_loop
            self._loop_func_args = ()

    def run(self):
        """Record data from queue to buffer."""
        self._can_quit.clear()
        with self._lock:
            self._must_write = True
        self._loop_func(*self._loop_func_args)
        self._can_quit.set()

    def _sleep_loop(self, period):
        while self._must_write:
            sleep(period)
            self.save_func()

    def _event_loop(self):
        while self._must_write:
            self._have_data.clear()
            self._have_data.wait()
            self.save_func()

    def write_data(self):
        """Ask thread to write data."""
        self._have_data.set()

    def stop(self):
        """Stop thread."""
        with self._lock:
            self._must_write = False
        self._can_quit.wait()
        self.join()


def _check_write_on(write_on):
    values = ['time', 'data']
    if write_on not in values:
        raise RuntimeError("write_on must be in {}".format(values))



import unittest
from PyQt5 import QtCore
from appteka.pyqt import testing
from appteka.pyqtgraph.waveform import Waveform


class TestWaveform(unittest.TestCase):
    """Test suite for Waveform widget."""
    def test_xlabel(self):
        app = testing.TestApp(self)

        w = Waveform(xlabel="Time [sec]")

        app(w, [
            "x label is 'Time [sec]'",
        ])

    def test_time_axis_false(self):
        app = testing.TestApp(self)

        w = Waveform(time_axis=False)
        w.update_data([0, 1, 2, 3], [1, 2, 1, 2])

        app(w, [
            "x values are usual numbers from 0 to 3",
        ])

    def test_time_axis_true(self):
        app = testing.TestApp(self)

        w = Waveform(None, time_axis=True)
        w.update_data([0, 1, 2, 3], [1, 2, 1, 2])

        app(w, [
            "x values are time values",
        ])

    def test_scaling(self):
        app = testing.TestApp(self)

        w = Waveform()
        w.update_data([0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [1, 2, 0, 3, 1, 2, 3, 4, 1, 3])

        app(w, [
            "both axis scaling with mouse wheel",
            "x-scaling with CONTROL pressed",
            "y-scaling with SHIFT pressed",
        ])


class TestWaveform_Animation(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.counter = 0

    def test_it(self):
        app = testing.TestApp(self)

        w = Waveform()

        def rotate():
            ts = [self.counter+i for i in range(10)]
            xs = [t for t in ts]
            w.update_data(ts, xs)
            self.counter = self.counter + 1

        timer = QtCore.QTimer()
        timer.setInterval(500)
        timer.timeout.connect(rotate)
        self.counter = 0
        timer.start()

        app(w, [
            "x values grow",
            "y values grow",
        ])



import unittest
from appteka.pyqt import testing
from appteka.pyqtgraph.waveform import MultiWaveform


class TestMultiWavefrom(unittest.TestCase):
    """MultiWavefrom."""
    def test_top_axis(self):
        """Switch on top axis."""
        app = testing.TestApp(self)

        w = MultiWaveform()

        w.add_plot('a', title='plot A')

        app(w, [
            "There is top axis",
        ])

    def test_scaling(self):
        """Scaling with keys CONTROL and SHIFT."""
        app = testing.TestApp(self)

        w = MultiWaveform()

        w.add_plot('a', title='plot A')
        w.update_data('a',
                      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [1, 2, 0, 3, 1, 2, 3, 4, 1, 3])

        w.add_plot('b', title='plot B')
        w.update_data('b',
                      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [2, 1, 1, 1, 4, 6, 2, 3, 4, 3])

        app(w, [
            "both axis scaling with mouse wheel",
            "x-scaling with CONTROL pressed",
            "y-scaling with SHIFT pressed",
        ])

    def test_set_title(self):
        """Change title of given plot."""
        app = testing.TestApp(self)

        w = MultiWaveform()
        w.add_plot('a', title='Title one')
        w.update_data('a', [0, 1, 2, 3], [1, 2, 1, 2])
        w.set_title(plot_key='a', value='Frequency: 50.123')

        app(w, ["Title is 'Frequency: 50.123'"])

    def test_color(self):
        app = testing.TestApp(self)

        w = MultiWaveform()
        w.add_plot('a')
        w.set_plots_color((255, 0, 0))
        w.update_data('a', [0, 1, 2, 3], [1, 2, 1, 2])

        app(w, ["Red plot"])



import unittest
from PyQt5 import QtCore
from appteka.pyqt import testing
from appteka.pyqtgraph import phasor


class TestPhasorDiagram(unittest.TestCase):
    def test_add_phasor(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1)
        app(d, ["White phasor in first quadrant"])

    def test_update_phasor(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1)
        d.update_phasor('ph-1', 80, 2)
        app(d, ["Phasor in second quadrant"])

    def test_phasor_color(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1, (255, 0, 0))
        app(d, ["Red phasor in first quadrant"])

    def test_three_phasors(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram(end='arrow')
        d.set_range(100)
        d.add_phasor('ph-1', 80, 0, (255, 0, 0))
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))

        app(d, [
            "3 phasors: red, green and blue",
            "About 120 degrees between phasors",
        ])

    def test_three_phasors_rotated(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()
        d.set_range(100)
        d.add_phasor('ph-1', 80, 0, (255, 0, 0))
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))
        d.update_phasor('ph-1', 80, 1)
        d.update_phasor('ph-2', 80, 1 + 2 * 3.1415 / 3)
        d.update_phasor('ph-3', 80, 1 - 2 * 3.1415 / 3)

        app(d, [
            "3 phasors: red, green and blue",
            "About 120 degrees between phasors",
            "Red phasor has angle about 1 radian",
        ])

    def test_width_of_phasors(self):
        app = testing.TestApp(self)

        # Given phasor diagram
        d = phasor.PhasorDiagram()

        # When add two phasors
        # And widths of phasors are set to be significantly differ
        d.add_phasor('ph-1', color=(255, 255, 255), width=1)
        d.add_phasor('ph-2', color=(255, 255, 255), width=4)
        d.update_phasor('ph-1', 100, 1)
        d.update_phasor('ph-2', 100, 2)
        d.set_range(100)

        # Then widths of phasors are differ
        app(d, ["Widths of phasors are differ"])


class TestPhasorDiagram_Range(unittest.TestCase):
    def test_range_is_two(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.set_range(2)
        app(d, ["Range is 2"])

    def test_change_range(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.set_range(2)
        d.set_range(4)
        app(d, ["Range is 4"])

    def test_range_to_phasor(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.add_phasor('ph-1', color=(255, 255, 0))
        d.update_phasor('ph-1', 1, 1)
        d.update_phasor('ph-1', 100, 1)
        d.set_range(100)
        app(d, ["Grid corresponds to phasor"])


class TestPhasorDiagram_Animation(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.counter = 0

    def test_three_phasors_animation(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram(end='arrow')
        d.add_phasor('ph-1', color=(255, 0, 0))
        d.add_phasor('ph-2', color=(0, 255, 0))
        d.add_phasor('ph-3', color=(0, 0, 255))
        d.show_legend()

        def rotate():
            a = 2 * 3.1415 / 3
            sh = self.counter / 200
            ash = self.counter / 10
            d.update_phasor('ph-1', ash + 10, sh)
            d.update_phasor('ph-2', 10, sh + a)
            d.update_phasor('ph-3', 10, sh - a)
            d.set_range(ash + 10)
            self.counter = self.counter + 1

        timer = QtCore.QTimer()
        timer.setInterval(10)
        timer.timeout.connect(rotate)
        self.counter = 0
        timer.start()

        app(d, [
            "Phasors smoothly rotating",
            "Amplitude of red phasor grows",
        ])


class TestPhasorDiagram_Legend(unittest.TestCase):
    def test_legend(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()
        d.add_phasor('ph-1', 80, 0, (255, 0, 0), width=3)
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))
        d.show_legend()
        d.set_range(80)

        app(d, [
            "Legend OK",
            "Lines in legend have different widths",
        ])

    def test_legend_prefer_names(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()
        d.add_phasor(0, amp=80, phi=0, color=(255, 0, 0), name="Ua")
        d.add_phasor(1, amp=80, phi=2*3.14/3, color=(0, 255, 0), name="Ub")
        d.add_phasor(2, amp=80, phi=-2*3.14/3, color=(0, 0, 255), name="Uc")
        d.show_legend()
        d.set_range(80)

        app(d, ["Legend: Ua, Ub, Uc"])

    def test_show_legend_twice(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()
        d.set_range(1)
        d.add_phasor(0, 1, 0)
        d.show_legend()
        d.show_legend()

        app(d, ["Legend OK"])


class TestPhasorDiagram_Clearing(unittest.TestCase):
    def test_clear_empty(self):
        app = testing.TestApp(self)
        d = phasor.PhasorDiagram()
        d.remove_phasors()
        app(d, ["Grid OK"])

    def test_clear_and_show_legend(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()

        d.set_range(80)
        d.add_phasor(0, amp=80, phi=0, color=(255, 0, 0), name="Ua")
        d.add_phasor(1, amp=80, phi=0, color=(0, 255, 0), name="Ub")
        d.show_legend()
        d.remove_phasors()

        d.add_phasor(0, amp=80, phi=0, name="Ua")
        d.show_legend()

        app(d, ["Legend OK"])


class TestPhasorDiagram_Visibility(unittest.TestCase):
    def test_set_invisible(self):
        app = testing.TestApp(self)

        d = phasor.PhasorDiagram()
        d.set_range(2)
        d.add_phasor(0, 1, 0, color=(255, 0, 0))
        d.add_phasor(1, 2, 1, color=(0, 255, 0))
        d.add_phasor(2, 2, 1, color=(0, 0, 255))
        d.show_legend()

        d.set_phasor_visible(1, False)
        d.set_phasor_visible(2, False)
        d.set_phasor_visible(2, True)
        d.set_phasor_visible(3, False)

        app(d, [
            "2 phasors in diagram",
            "3 items in legend",
        ])


class TestPhasorDiagram_Deprecation(unittest.TestCase):
    def test_size_arg(self):
        testing.TestApp(self)
        with self.assertWarns(FutureWarning):
            phasor.PhasorDiagram(size=100)

    def test_end_arg(self):
        testing.TestApp(self)
        with self.assertWarns(FutureWarning):
            phasor.PhasorDiagram(end='arrow')



import unittest
from appteka.distrib import micro_version, minor_version, major_version


class Test_mirco_version(unittest.TestCase):
    def test_regular_case(self):
        ver = "0.2.11"
        self.assertEqual(micro_version(ver), 11)

    def test_2_parts_so_zero_micro(self):
        ver = "0.2"
        self.assertEqual(micro_version(ver), 0)

    def test_4_parts(self):
        ver = "0.2.1.3"
        self.assertEqual(micro_version(ver), 1)


class Test_minor_version(unittest.TestCase):
    def test_regular_case(self):
        ver = "0.2.11"
        self.assertEqual(minor_version(ver), 2)

    def test_1_part(self):
        ver = "15"
        self.assertEqual(minor_version(ver), 0)

    def test_2_part(self):
        ver = "0.2"
        self.assertEqual(minor_version(ver), 2)

    def test_4_parts(self):
        ver = "0.2.1.3"
        self.assertEqual(minor_version(ver), 2)


class Test_major_version(unittest.TestCase):
    def test_regular_case(self):
        ver = "0.2.11"
        self.assertEqual(major_version(ver), 0)



# appteka - helpers collection

# Copyright (C) 2018-2021 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Implementation of the phasor diagram."""

from math import degrees
from warnings import warn
import cmath
import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtWidgets


DEFAULT_WIDGET_SIZE = QtCore.QSize(500, 500)

CIRCLES_NUM = 6
DEFAULT_COLOR = (255, 255, 255)
DEFAULT_WIDTH = 1


class PhasorDiagram(pg.PlotWidget):
    """Widget for plotting phasor diagram.

    Parameters
    ----------
    parent: object
        Parent object
    """

    def __init__(self, parent=None, size=None, end=None):
        super().__init__(parent)

        if size is not None:
            warn("size arg is deprecated and ignored", FutureWarning)

        if end is not None:
            warn("end arg is deprecated and ignored", FutureWarning)

        self.setAspectLocked(True)
        self.addLine(x=0, pen=0.2)
        self.addLine(y=0, pen=0.2)
        self.showAxis('bottom', False)
        self.showAxis('left', False)

        policy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred,
            QtWidgets.QSizePolicy.Preferred,
        )
        policy.setHeightForWidth(True)
        self.setSizePolicy(policy)

        self.__build_grid()
        self.__build_labels()

        self.set_range(1)

        self.setMouseEnabled(x=False, y=False)
        self.disableAutoRange()
        self.plotItem.setMenuEnabled(False)
        self.hideButtons()

        self.__legend = None
        self.__phasors = {}
        self.__items = {}
        self.__names = {}

    def set_range(self, value):
        """Set range of diagram."""
        self.__range = value
        self.__update_grid()
        self.__update_labels()

    def add_phasor(self, key, amp=0, phi=0,
                   color=DEFAULT_COLOR, width=DEFAULT_WIDTH,
                   name=None):

        """Add phasor to the diagram."""

        line = self.plot(
            pen=pg.mkPen(color, width=width),
        )

        arr = pg.ArrowItem(
            tailLen=0,
            tailWidth=1,
            pen=pg.mkPen(color, width=width),
            headLen=width+4,
            brush=None,
        )
        self.addItem(arr)

        self.__items[key] = {'line': line, 'arr': arr}
        self.__names[key] = name

        self.update_phasor(key, amp, phi)

    def set_phasor_visible(self, key, value=True):
        """Hide or show phasor."""
        if key not in self.__items:
            return

        for item in self.__items[key]:
            self.__items[key][item].setVisible(value)

    def update_phasor(self, key, amp, phi):
        """Change phasor value."""
        self.__phasors[key] = (amp, phi)
        self.__update()

    def remove_phasors(self):
        """Remove phasors and legend."""

        for key in self.__items:
            for subkey in self.__items[key]:
                self.removeItem(self.__items[key][subkey])

        self.__items = {}
        self.__names = {}
        self.__phasors = {}

        if self.__legend is not None:
            self.__legend.clear()
            self.removeItem(self.__legend)

        self.__legend = None

    def show_legend(self):
        """Show legend."""

        if self.__legend:
            return

        self.__legend = self.plotItem.addLegend()
        for key in self.__items:
            name = self.__names[key]
            if name:
                self.plotItem.legend.addItem(
                    self.__items[key]['line'], name)
            else:
                self.plotItem.legend.addItem(
                    self.__items[key]['line'], key)

    def __update(self):
        for key in self.__phasors:
            phasor = self.__phasors[key]
            compl = cmath.rect(*phasor)
            x = compl.real
            y = compl.imag

            items = self.__items[key]

            items['line'].setData([0, x], [0, y])

            arr = items['arr']
            arr.setStyle(angle=180 - degrees(phasor[1]))
            arr.setPos(x, y)

    def __build_grid(self):
        self.circles = []
        for _ in range(CIRCLES_NUM):
            circle = pg.QtGui.QGraphicsEllipseItem()
            circle.setPen(pg.mkPen(0.2))
            self.circles.append(circle)
            self.addItem(circle)

    def __update_grid(self):
        for i in range(CIRCLES_NUM):
            rad = (i + 1) * self.__range / CIRCLES_NUM
            self.circles[i].setRect(-rad, -rad, rad*2, rad*2)

        self.setRange(QtCore.QRectF(-rad, rad, 2*rad, -2*rad))

    def __build_labels(self):
        self.labels = []
        for _ in range(2):
            label = pg.TextItem()
            self.labels.append(label)
            self.addItem(label)

    def __update_labels(self):
        self.labels[0].setText("{}".format(self.__range / 2))
        self.labels[0].setPos(0, self.__range / 2)
        self.labels[1].setText("{}".format(self.__range))
        self.labels[1].setPos(0, self.__range)

    def sizeHint(self):
        # pylint: disable=invalid-name,no-self-use,missing-docstring
        return DEFAULT_WIDGET_SIZE

    def heightForWidth(self, width):
        # pylint: disable=invalid-name,no-self-use,missing-docstring
        return width



import unittest
from time import sleep
import os
from appteka.io import QueuedWriter


TMP_FILE_NAME = "tmp.txt"


def convert_func(sample):
    """Convert function."""
    return '{}: {}\n'.format(*sample)


class TestQueuedWriter(unittest.TestCase):
    def test_on_timer(self):
        writer = QueuedWriter(write_on='time')
        writer.set_convert_func(convert_func)
        buf = open(TMP_FILE_NAME, 'w')
        writer.set_buff(buf)
        writer.start_thread()

        i = 0
        while i < 150:
            sample = (i, i*2)
            writer.add_data(sample)
            sleep(0.01)
            i += 1

        writer.stop_thread()
        buf.close()
        os.remove(TMP_FILE_NAME)

    def test_on_data(self):
        writer = QueuedWriter(write_on='data')
        writer.set_convert_func(convert_func)
        buf = open(TMP_FILE_NAME, 'w')
        writer.set_buff(buf)
        writer.start_thread()

        i = 0
        while i < 150:
            sample = (i, i*2)
            writer.add_data(sample)
            sleep(0.01)
            i += 1

        writer.stop_thread()
        buf.close()
        os.remove(TMP_FILE_NAME)

    def test_wront_write_on(self):
        with self.assertRaises(RuntimeError):
            QueuedWriter(write_on='dollar fault')



# appteka - helpers collection

# Copyright (C) 2018-2021 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The tool for visual testing of widgets."""

import sys
from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
from appteka.pyqt import gui


class TestApp:
    """An application for displaying a widget that has been set to a
    certain state and asking questions that express the requirements
    of the test case.

    Parameters
    ----------
    context: unittest.TestCase
        The class in that the test case should being ran.
    """

    __app = None

    def __init__(self, context):
        if not TestApp.__app:
            TestApp.__app = QtWidgets.QApplication(sys.argv)

        self.answer = False
        self.dialog = _Dialog(self)
        self.context = context

    def __call__(self, widget, assertions):
        self.dialog.set_widget(widget)
        self.dialog.set_assertions(assertions)

        self.dialog.show()
        TestApp.__app.exec()

        self.context.assertTrue(self.answer)


class _Dialog(QtWidgets.QDialog):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self.app = app
        self.__make_gui()

    def __make_gui(self):
        main_layout = QtWidgets.QVBoxLayout(self)

        self.widget_layout = gui.add_sublayout(main_layout, "h")

        self.assert_layout = gui.add_sublayout(main_layout, "h")
        self.label_assert = gui.add_label("", self.assert_layout)
        self.label_assert.setWordWrap(True)

        self.button_layout = gui.add_sublayout(main_layout, "h")
        self.button_no = gui.add_button("No", self.__on_button_no,
                                        self.button_layout)
        self.button_yes = gui.add_button("Yes", self.__on_button_yes,
                                         self.button_layout)
        self.button_yes.setFocus(Qt.ActiveWindowFocusReason)

    def set_widget(self, widget):
        """Set widget to be shown."""
        self.widget_layout.addWidget(widget)

    def set_assertions(self, assertions):
        """Set assertions to be printed in dialog."""
        lines = ""
        for asr in assertions:
            lines += "- {}\n".format(asr)
        self.label_assert.setText(lines)

    def __on_button_no(self):
        self.app.answer = False
        self.reject()

    def __on_button_yes(self):
        self.app.answer = True
        self.accept()



import unittest
from appteka.pyqt import testing
from appteka.pyqt.code_text_edit import CodeTextEdit


class TestCodeTextEdit(unittest.TestCase):
    """Tests for CodeTextEdit"""

    def test_text(self):
        app = testing.TestApp(self)

        e = CodeTextEdit()

        code = ''
        code += '{\n'
        code += '  "a": 1,\n'
        code += '  "b": 2\n'
        code += '}'
        e.set_text(code)

        app(e, [
            "Some text is printed",
            "Lines numbered",
            "Current line is highlighted",
            "Font is monospace",
        ])
